//Wiznet HTML server using official C library
//RJA deriving from w5x00_http_server.c
/**
 * Copyright (c) 2021 WIZnet Co.,Ltd
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

struct __using("Resources.Spin2") Spin2Res; //RJA: //Visual Studio will show an error here, but it's OK

#include "Platform.h"


#include "wizchip_conf.h"
#include "w5x00_spi.h"
#include "w5500.h"
#include "socket.h"
#include "httpParser.h"
#include "httpUtil.h"
#include "dhcp.h"
#include "httpServer.h"

#include "w5500.c"
#include "w5x00_spi.c"
#include "wizchip_conf.c"
#include "socket.c"
#include "httpParser.c"
#include "httpUtil.c"
#include "dhcp.c"
#include "httpServer.c"



/* Buffer */
#define ETHERNET_BUF_MAX_SIZE (1024 * 2)

/* Socket */
#define HTTP_SOCKET_MAX_NUM 4

/* Socket */
#define SOCKET_DHCP 7
#define SOCKET_DNS 6

/* Retry count */
#define DHCP_RETRY_COUNT 5
#define DNS_RETRY_COUNT 5

/**
 * ----------------------------------------------------------------------------------------------------
 * Variables
 * ----------------------------------------------------------------------------------------------------
 */
 /* Network */  //RJA:  This kind of structure initializion isn't currently working in FlexC, doing it manually below
static wiz_NetInfo g_net_info; /* =
{
    .mac = {0x00, 0x08, 0xDC, 0x12, 0x34, 0x56}, // MAC address
    .ip = {192, 168, 1, 2},                     // IP address  192.168.1.2
    .sn = {255, 255, 255, 0},                    // Subnet Mask
    .gw = {192, 168, 1, 1}, //RJA {192, 168, 11, 1},                     // Gateway
    .dns = {192, 168, 1, 1}, //RJA {8, 8, 8, 8},                         // DNS server
    .dhcp = NETINFO_STATIC  //NETINFO_DHCP //RJA                      // DHCP enable/disable
};
*/

static uint8_t g_ethernet_buf[ETHERNET_BUF_MAX_SIZE]; //RJA = {


/* HTTP */
static uint8_t g_http_send_buf[ETHERNET_BUF_MAX_SIZE]; //RJA = {

static uint8_t g_http_recv_buf[ETHERNET_BUF_MAX_SIZE];  //RJA = {

static uint8_t g_http_socket_num_list[HTTP_SOCKET_MAX_NUM] = { 0, 1, 2, 3 };


/* DHCP */
static void wizchip_dhcp_init(void);
static void wizchip_dhcp_assign(void);
static void wizchip_dhcp_conflict(void);
/* DHCP */
static uint8_t g_dhcp_get_ip_flag = 0;


void Setup()
{
    g_ethernet_buf[0] = 0;
    g_http_send_buf[0] = 0;
    g_http_recv_buf[0] = 0;
}

void Loop()
{
    printf("Loop...\n", f);
}




int main()
{

    printf("Starting\n");

    g_net_info.mac[0] = 0x00;
    g_net_info.mac[1] = 0x08;
    g_net_info.mac[2] = 0xDC;
    g_net_info.mac[3] = 0x12;
    g_net_info.mac[4] = 0x34;
    g_net_info.mac[5] = 0x56;
    g_net_info.ip[0] = 192;
    g_net_info.ip[1] = 168;
    g_net_info.ip[2] = 1;
    g_net_info.ip[3] = 249;
    g_net_info.sn[0] = 255;
    g_net_info.sn[1] = 255;
    g_net_info.sn[2] = 255;
    g_net_info.sn[3] = 0;
    g_net_info.gw[0] = 192;
    g_net_info.gw[1] = 168;
    g_net_info.gw[2] = 1;
    g_net_info.gw[3] = 1;
    g_net_info.dns[0] = 192;
    g_net_info.dns[1] = 168;
    g_net_info.dns[2] = 1;
    g_net_info.dns[3] = 1;
    g_net_info.dhcp = NETINFO_STATIC;  //NETINFO_DHCP //RJA                      // DHCP enable/disable

    printf(" IP          : %d.%d.%d.%d\n", g_net_info.ip[0], g_net_info.ip[1], g_net_info.ip[2], g_net_info.ip[3]);

    Setup();

    /* Initialize */
    uint8_t i = 0;
    uint8_t retval = 0;
    uint8_t dhcp_retry = 0;

    printf("wizchip_spi_initialize\n");
    wizchip_spi_initialize();
    printf("wizchip_cris_initialize\n");
    wizchip_cris_initialize();

    printf("wizchip_reset\n");
    wizchip_reset();
    printf("wizchip_initialize\n");
    wizchip_initialize();
    printf("wizchip_check\n");
    wizchip_check();
    printf("Check complete.");

    if (g_net_info.dhcp == NETINFO_DHCP) // DHCP
    {
        wizchip_dhcp_init();
    }
    else // static
    {
        network_initialize(g_net_info);
    }
    


    
    //RJA FlexC didn't seem to initialize these correctly:
    g_http_socket_num_list[0] = 0;
    g_http_socket_num_list[1] = 1;
    g_http_socket_num_list[2] = 2;
    g_http_socket_num_list[3] = 3;

    httpServer_init(g_http_send_buf, g_http_recv_buf, HTTP_SOCKET_MAX_NUM, g_http_socket_num_list);  

    /* Get network information */
   print_network_information(g_net_info);
   printf("starting register.\n");

    /* Register web page */
   reg_httpServer_webContent("index.html", index_page);
   reg_httpServer_webContent2("beany.png", Spin2Res.Get_BMPAddress(1), Spin2Res.Get_BMPSize(1));  //RJA added another way to register content
   reg_httpServer_webContent2("bitmap2.bmp", Spin2Res.Get_BMPAddress(2), Spin2Res.Get_BMPSize(2));  //RJA added another way to register content
   reg_httpServer_webContent2("test.html", Spin2Res.Get_Text1Address(), Spin2Res.Get_Text1Size());

   

   if (g_net_info.dhcp == NETINFO_DHCP) // DHCP
   {
       //DHCP
       /* Assigned IP through DHCP */
       g_dhcp_get_ip_flag = 0;
       while (g_dhcp_get_ip_flag == 0)
       {
           if (g_net_info.dhcp == NETINFO_DHCP)
           {
               retval = DHCP_run();

               if (retval == DHCP_IP_LEASED)
               {
                   if (g_dhcp_get_ip_flag == 0)
                   {
                       printf(" DHCP success\n");
                       printf(" IP          : %d.%d.%d.%d\n", g_net_info.ip[0], g_net_info.ip[1], g_net_info.ip[2], g_net_info.ip[3]);  //RJA
                       g_dhcp_get_ip_flag = 1;
                   }
               }
               else if (retval == DHCP_FAILED)
               {
                   g_dhcp_get_ip_flag = 0;
                   dhcp_retry++;

                   if (dhcp_retry <= DHCP_RETRY_COUNT)
                   {
                       printf(" DHCP timeout occurred and retry %d\n", dhcp_retry);
                   }
               }

               if (dhcp_retry > DHCP_RETRY_COUNT)
               {
                   printf(" DHCP failed\n");

                   DHCP_stop();

                   while (1)
                       ;
               }

               //wizchip_delay_ms(1000); // wait for 1 second
               _waitms(1000); //RJA
           }
       }
   }

    /* Infinite loop */
    while (1)
    {

        
        /* Run HTTP server */
        for (i = 0; i < HTTP_SOCKET_MAX_NUM; i++)
        {
            httpServer_run(i);


        }


    }

 


}


/* DHCP */
static void wizchip_dhcp_init(void)
{
    printf(" DHCP client running\n");

    DHCP_init(SOCKET_DHCP, g_ethernet_buf);

    reg_dhcp_cbfunc(wizchip_dhcp_assign, wizchip_dhcp_assign, wizchip_dhcp_conflict);
}

static void wizchip_dhcp_assign(void)
{
    getIPfromDHCP(g_net_info.ip);
    getGWfromDHCP(g_net_info.gw);
    getSNfromDHCP(g_net_info.sn);
    getDNSfromDHCP(g_net_info.dns);

    g_net_info.dhcp = NETINFO_DHCP;

    /* Network initialize */
    network_initialize(g_net_info); // apply from DHCP

    print_network_information(g_net_info);
    printf(" DHCP leased time : %ld seconds\n", getDHCPLeasetime());
}

static void wizchip_dhcp_conflict(void)
{
    printf(" Conflict IP from DHCP\n");

    // halt or reset or any...
    while (1)
        ; // this example is halt.
}

/* Timer */
static void repeating_timer_callback(void)
{
    g_msec_cnt++;

    if (g_msec_cnt >= 1000 - 1)
    {
        g_msec_cnt = 0;

        DHCP_time_handler();
        DNS_time_handler();
    }
}

